<?php
namespace Controller;

use Util\Utilities;
use Util\Messages;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;


class Comments extends \Generated\Controller\Comments
{

    /**
     * @var \Util\Utilities
     */
    private $utilities;
    private $taskService;

    /**
     * @var \Service\Attachment
     */
    private $attachmentService;
    /**
     * @var \Service\Comment
     */
    private $commentService;

    public function __construct(Manager $dataSourceManager, \Util\Utilities $utilities, \Service\Task $taskService,
                                \Service\Comment $commentService, \Service\Attachment $attachmentService)
    {
        $this->utilities = $utilities;
        $this->taskService = $taskService;
        parent::__construct($dataSourceManager);
        $this->commentService = $commentService;
        $this->attachmentService = $attachmentService;
    }

    /**
     * Rewriting our find() method and setting a condition to FilterCriteria
     *
     * @param DTO\Request\Filter $request A DTO\Request with a FilterCriteria
     * @param DTO\Response $response A DTO\Response to be added with DataSets
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response, $showMessage = false)
    {
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->utilities->getCurrentOrganizationId());
        parent::find($request, $response, $showMessage);
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = false)
    {
        try {
            $status = $request->getDataSet()->getRows();

            if (!empty($status)) {
                $issueId = $status[0]['TASK'];
                $comment = $status[0]['DESCRIPTION'];
                $document = empty($status[0]['DOCUMENT'])? null :$status[0]['DOCUMENT'];
                foreach ($status as &$eachStatus) {
                    $eachStatus['ORGANIZATION'] = $this->utilities->getCurrentOrganizationId();
                    $eachStatus['CREATED_BY'] = $this->utilities->getCurrentUserId();
                    $eachStatus['ID'] = null;
                    $eachStatus['PROJECT'] = null;
                }
                $dataSet = new DTO\Request\DataSet(new \Zeedhi\Framework\DataSource\DataSet($request->getDataSet()->getDataSourceName(), $status),
                    $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($dataSet, $response, $showMessage);
                if($document) {
                    $commentId = $response->getDataSets($request->getDataSet()->getDataSourceName())[0]->getRows()[0]["ID"];
                    $this->attachmentService->addDocumentComment($commentId, $document);
                }
                $control = $this->commentService->getControlByCommentTypeId($status[0]["COMMENT_TYPE_ID"]);
                if(!isset($status[0]["HOMOLOGATED"]) && $control != Entities\CommentType::TYPE_REFUSED) {
                    $this->commentService->sendsMailAdvisingAboutNewIssueComment($issueId, $comment);
                }
                $this->commentService->viewCommentsSaveData($issueId);
                $this->commentService->setCurrentTaskStatus($issueId);
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Erro inesperado ' . $e->getMessage()));
            throw $e;
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        try {
            $status = $request->getDataSet()->getRows();
            if (!empty($status)) {
                foreach ($status as &$eachStatus) {
                    if(empty($eachStatus['__is_new'])){
                        $eachStatus['__is_new'] = false;
                    }
                };
                parent::delete($request, $response, $showMessage);
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Erro inesperado ' . $e->getMessage()));
            throw $e;
        }
    }
    
    public function saveAta(DTO\Request $request, DTO\Response $response, $showMessage = false)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            $taskId = null;
            if (!empty($rows)) {
                foreach ($rows as &$row) {
                    if(empty($row['__is_new'])){
                        $row['__is_new'] = false;
                    }else{
                        $row['ORGANIZATION'] = $this->utilities->getCurrentOrganizationId();
                        $row['CREATED_BY'] = $this->utilities->getCurrentUserId();
                        $row['ID'] = null;
                        $row['PROJECT'] = null;
                    }
                    $taskId = $row['TASK'];
                };
                parent::save(new DTO\Request\DataSet(new \Zeedhi\Framework\DataSource\DataSet($request->getDataSet()->getDataSourceName(), $rows),
                    $request->getMethod(), $request->getRoutePath(), $request->getUserId()), $response, $showMessage);
                $reload = $this->taskService->setInProgressMeetingWhenBelatedByScheduleId($taskId);
                $response->addDataSet(new DataSet('reload', array('RELOAD' =>$reload)));
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Erro inesperado ' . $e->getMessage()));
            throw $e;
        }
    }

    /**
     * Used in devops
     * @param DTO\Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function commentsIssuesRelease(DTO\Request\DataSet $request, Response $response)
    {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $errorLog = array();
            foreach ($releaseData as $row) {
                $releaseId = $row['RELEASE_ID'];
                $version = empty($row['VERSION'])? null :$row['VERSION'];
                $errorLog = $this->commentService->sendCommentsIssuesRelease($releaseId,$version);
            }
            $response->addDataset(new DataSet('errorLog', $errorLog));

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Used in devops
     * @param DTO\Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function commentsIssuesSubRelease(DTO\Request\DataSet $request, Response $response)
    {
        try {
            $releaseData = $request->getDataSet()->getRows();
            $errorLog = array();
            foreach ($releaseData as $row) {
                $releaseId = $row['SUBRELEASE_ID'];
                $version = empty($row['VERSION'])? null :$row['VERSION'];
                $errorLog = $this->commentService->sendCommentsIssuesSubRelease($releaseId,$version);
            }
            $response->addDataset(new DataSet('errorLog', $errorLog));

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function searchForSingleComments(Request\Filter $request,Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions()[0];
            $issues = array();
            foreach($filters["value"] as $filter){
                array_push($issues,$filter["ID"]);
            }
            $singleIssues = $errorLog = $this->commentService->setSingleCommentsAsRead($issues);

            $response->addDataset(new DataSet('lastDate', $singleIssues));

        } catch (\Exception $e) {
            throw $e;
        }
    }


}