<?php
namespace Controller;

use Util\InstanceProvider;
use Util\Messages;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class DocumentTemplate extends Simple {

    public function findDocumentTemplate(Request\Filter $request, Response $response) {
        try{
            $filter = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $documentTemplates = InstanceProvider::getService()->getDocumentTemplate()->findDocumentTemplate($filter,$page,$pageSize);
            Utilities::setDataSetDefault($request,$response,$documentTemplates);
        }catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveDocumentTemplate(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $documentTemplateId = empty($row['ID']) ? null : $row['ID'];
            $name = $row['NAME'];
            $type = $row['TYPE'];
            $document = empty($row['DOCUMENT']) ? null : $row['DOCUMENT'];
            $documentTemplateId = InstanceProvider::getService()->getDocumentTemplate()
                ->saveDocumentTemplate($documentTemplateId, $name, $type, $document);
            Utilities::setDataSetDefault($request,$response,[['ID'=>$documentTemplateId]]);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function removeDocumentTemplate(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $documentTemplateId = empty($row['ID']) ? null : $row['ID'];
           InstanceProvider::getService()->getDocumentTemplate()
                ->removeDocumentTemplate($documentTemplateId);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function generateIssueBudget(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $documentTemplateId = empty($row['DOCUMENT_TEMPLATE_ID']) ? null : $row['DOCUMENT_TEMPLATE_ID'];
            $issueId = empty($row['ISSUE_ID']) ? null : $row['ISSUE_ID'];
            InstanceProvider::getService()->getDocumentTemplate()
                ->generateBudget($documentTemplateId,$issueId);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }
}