<?php
/**
 * Created by PhpStorm.
 * User: eugeniofonseca
 */

namespace Service;


use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities;

class Budget {

    public function budgetItem($conditions,$page,$pageSize,$notClassificationId = null) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditionsSQL = Utilities::createQueryFiltered($conditions);
        $budgetItems = InstanceProvider::getRepository()->getBudgetItem()->budgetItem(
            InstanceProvider::getEnvironment()->getCurrentOrganizationId(), $conditionsSQL, $limit,
            $offset,$notClassificationId);
        return $budgetItems;
    }

    public function findIssueBudget($issueId,$nameProposal,$nameDiscount,$namePercent)
    {
        return InstanceProvider::getRepository()->getBudget()
            ->findIssueBudget($issueId,$nameProposal,$nameDiscount,$namePercent);
    }

    public function issueBudgetSave($rows){

        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();
            foreach ($rows as $row) {
                $issueId = $row['ISSUE_ID'];
                $budgetItemId = $row['BUDGET_ITEM_ID'];
                $proposalQuantity = $row['PROPOSAL_QUANTITY'];
                $proposalValue = $row['PROPOSAL_VALUE'];
                $unitMeasure = $row['UNIT_MEASURE'];
                InstanceProvider::getService()->getBudget()->issueBudgetRowSave($issueId, $budgetItemId,
                    $proposalQuantity, $proposalValue, $unitMeasure);
            }
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function issueBudgetRowSave($issueId,$budgetItemId,$proposalQuantity,$proposalValue)
    {

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $instanceFactory = InstanceProvider::getFactory();
        /** @var Entities\Task $issue */
        $issue = $instanceRepository->getTask()->find($issueId);
        /** @var Entities\Budget $budget */
        $budget = $instanceRepository->getBudget()->findOneBy(array('task' => $issue));
        $budget = $instanceFactory->getBudget()->save($budget,$issue);
        if($budgetItemId == 'XXXX'){
            $budget->setDiscountPercent($proposalQuantity);
            $budget->setDiscountValue($proposalValue);
            $entityManager->persist($budget);
            $entityManager->flush();
        }else{
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = $instanceRepository->getBudgetItem()->find($budgetItemId);
            /** @var Entities\BudgetItemBudget $budgetItemBudget */
            $budgetItemBudget = $instanceRepository->getBudgetItemBudget()
                ->findOneBy(array( 'budgetItem' => $budgetItem, 'budget' => $budget, ));
            if($proposalQuantity){
                $valueItem = $budgetItem->getValue();
                $unitMeasure = $budgetItem->getUnitMeasure();
                $budgetItemBudget = $instanceFactory->getBudgetItemBudget()->save($budgetItemBudget,$budget,$budgetItem,
                    $proposalQuantity,$valueItem,$unitMeasure);
                $entityManager->persist($budgetItemBudget);
                $entityManager->flush();
            }else if($budgetItemBudget){
                $entityManager->remove($budgetItemBudget);
                $entityManager->flush();
            }
        }
    }

    public function classificationBudgetItem($classificationId,$conditions,$page,$pageSize) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditionsSQL = Utilities::createQueryFiltered($conditions);
        $budgetItems = InstanceProvider::getRepository()->getClassificationBudgetItem()->classificationBudgetItem($classificationId,
            $conditionsSQL, $limit, $offset);
        return $budgetItems;
    }

    public function budgetItemSave($id,$name,$valueItem,$unitMeasure,$status) {
        $entityManager = InstanceProvider::getEntityManager();
        $budgetItem = null;
        if($id){
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
        }
        $valueItem = Utilities::fixBugSaveCurrencyMask($valueItem);
        $budgetItem = InstanceProvider::getFactory()->getBudgetItem()->save($budgetItem,$name, $valueItem, $unitMeasure,
            $status);
        $entityManager->persist($budgetItem);
        $entityManager->flush();
        return ['ID' => $budgetItem->getId()];
    }

    public function addBudgetItemInClassificationSave($budgetItemId,$classificationId) {
        $entityManager = InstanceProvider::getEntityManager();
        $classificationBudgetItemId = null;
        if($budgetItemId && $classificationId){
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($budgetItemId);
            /** @var Entities\Classification $classification */
            $classification = InstanceProvider::getRepository()->getClassification()->find($classificationId);
            $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()
                ->findOneBy(array('classification' => $classificationId,'budgetItem' => $budgetItemId,));
            if(!$classificationBudgetItem){
                $classificationBudgetItem = InstanceProvider::getFactory()->getclassificationBudgetItem()->create($budgetItem,$classification);

                $entityManager->persist($classificationBudgetItem);
                $entityManager->flush();
                $classificationBudgetItemId = $classificationBudgetItem->getId();
            }
        }
        return ['ID' => $classificationBudgetItemId];
    }

    public function budgetItemRemove($id) {
        $entityManager = InstanceProvider::getEntityManager();
        /** @var Entities\BudgetItem $budgetItem */
        $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
        $budgetItemBudget = InstanceProvider::getRepository()->getBudgetItemBudget()->findOneBy(array('budgetItem' => $budgetItem));

        if(!$budgetItemBudget){
            $classificationBudgetItems = InstanceProvider::getRepository()->getClassificationBudgetItem()
                ->findBy(array('budgetItem' => $budgetItem));

            foreach ($classificationBudgetItems as $classificationBudgetItem) {
                $entityManager->remove($classificationBudgetItem);
            }
            $entityManager->flush();

            $budgetItem->deleteEntity();
        }else{
            $budgetItem->setStatus(Entities\BudgetItem::IS_NOT_ACTIVE);
        }
        $entityManager->persist($budgetItem);
        $entityManager->flush();
    }

    public function classificationBudgetItemRemove($id) {
        $entityManager = InstanceProvider::getEntityManager();
        /** @var Entities\ClassificationBudgetItem $budgetItem */
        $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()->find($id);
        $entityManager->remove($classificationBudgetItem);
        $entityManager->flush();
    }
}