<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 23/04/2015
 * Time: 14:23
 */

namespace Service\Calendar;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Task;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskRelationship;
use Service\Calendar;
use Service\Calendar\Precursor\Strategy\PrecursorFactory;
use Service\Calendar\Restriction\Strategy\RestrictionFactory;
use Service\CalendarDetail;
use Service\TaskPrecursor;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface;

class TaskCalendar implements TaskCalendarImpl
{

    CONST DAY_WEEK_SUNDAY = 'SU';
    CONST DAY_WEEK_MONDAY = 'MO';
    CONST DAY_WEEK_TUESDAY = 'TU';
    CONST DAY_WEEK_WEDNESDAY = 'WE';
    CONST DAY_WEEK_THURSDAY = 'TH';
    CONST DAY_WEEK_FRIDAY = 'FR';
    CONST DAY_WEEK_SATURDAY = 'SA';

    CONST FORMAT_WEEK = 'w';
    CONST ADD_DAY = '+1 day';
    CONST REMOVE_DAY = '-1 day';

    CONST NAME_COMMITMENT_EXCEPTION = 'E';
    CONST NAME_COMMITMENT_WORK = 'W';

    CONST USES_DATA_CONTROL = 'Y';
    CONST NO_USES_DATA_CONTROL = 'N';

    CONST HOUR_DAY = 28800;

    public static $ARRAY_WEEK = array(
        self::DAY_WEEK_SUNDAY => 0,
        self::DAY_WEEK_MONDAY => 1,
        self::DAY_WEEK_TUESDAY => 2,
        self::DAY_WEEK_WEDNESDAY => 3,
        self::DAY_WEEK_THURSDAY => 4,
        self::DAY_WEEK_FRIDAY => 5,
        self::DAY_WEEK_SATURDAY => 6,
    );
    public static $ARRAY_WEEK_BR_LABEL = array(
        'DOM', 'SEG', 'TER', 'QUA', 'QUI', 'SEX', 'SAB',
    );
    /**
     * @var EntityManager $entityManager
     */
    private $entityManager;
    /**
     * @var \Service\Allocation
     */
    private $allocationService;
    /**
     * @var \Service\Calendar
     */
    private $calendarService;
    /**
     * @var \Zeedhi\DevOps\Model\ProjectManagement\Repositories\Project
     */
    private $projectRepository;


    public function __construct(EntityManager $entityManager, \Service\Allocation $allocationService,
                                \Service\Calendar $calendarService)
    {
        $this->entityManager = $entityManager;
        $this->allocationService = $allocationService;
        $this->calendarService = $calendarService;
        $this->projectRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project');
    }


    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task | \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $task
     * @return mixed
     */
    public function setDates($task)
    {
        if ($task->usesDataControl()) {
            $this->setDate($task);

            $this->setDateParent($task);

            $this->setDatesTaskRelationship($task);

            $hasChildren = $task->hasChildren();
            if ($hasChildren) {
                $task->setDuration($this->setDuration($task));
            }
        }
        $this->entityManager->persist($task);
        $this->entityManager->flush();

    }

    /**
     * @param TaskCalendarInterface $task
     * @param \DateTime $begin
     * @param $setAllocation
     * @return \DateTime
     */
    public function getDateEnd($task, \DateTime $begin, $setAllocation = true)
    {
        $begin = clone $begin;
        $duration = $task->getDuration() + $task->getDurationPrecursor() + $task->getLatency();
        if ($setAllocation) {
            $this->allocationService->deleteAllDailyAllocation($task);
        }
        $first = true;
        $begin = $this->getEndDateByCalendar($task, $begin, $first, $duration, false, $setAllocation);
        return $begin;
    }

    /**
     * @param TaskCalendarInterface $task
     * @param \DateTime $end
     * @param bool $setAllocation
     * @return \DateTime
     */

    public function getDateBegin($task, \DateTime $end, $setAllocation = true)
    {
        $end = clone $end;
        $duration = $task->getDuration();
        if ($setAllocation) {
            $this->allocationService->deleteAllDailyAllocation($task);
        }
        $first = true;
        $end = $this->getBeginDateByCalendar($task, $end, $first, $duration, false, $setAllocation);
        return $end;
    }

    /**
     * @param TaskCalendarInterface $task
     * @param $dateBeginMax
     * @param $dateEndMin
     * @return mixed
     */
    public function setDateParent($task, \DateTime $dateBeginMax = null, \DateTime $dateEndMin = null)
    {
        list($taskBegin, $taskEnd, $dateBeginMax, $dateEndMin) = $this->setDateByChildren($task, $dateBeginMax, $dateEndMin);
        if ($task->getTreeParent()) {
            if ($task->getTreeParent()->usesDataControl()) {
                $taskParent = $task->getTreeParent();
                $this->setDateParent($taskParent, $dateBeginMax, $dateEndMin);
                $this->entityManager->persist($taskParent);

            }
        }
        return array($taskBegin, $taskEnd, $dateBeginMax, $dateEndMin);

    }

    /**
     * @param $calendarDates
     * @param bool $first
     * @throws \Exception
     * @return \DateTime
     */
    public function getNextWorkingDay($calendarDates, $first = false)
    {

        foreach ($calendarDates as $key => $calendarDate) {
            $isAWorkDay = $calendarDate['TYPE'] == \Service\CalendarDetail::WORK_TYPE;
            if ($isAWorkDay) {
                if ($first) {
                    $first = false;
                } else {
                    return \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
                }
            }
        }

        throw new \Exception('date not found in CalendarDates');
    }

    /**
     * @param $calendarDates
     * @param bool $first
     * @throws \Exception
     * @return \DateTime
     */
    public function getPreviousWorkingDay($calendarDates, $first = false)
    {
        array_reverse($calendarDates);
        foreach ($calendarDates as $calendarDate) {
            if ($calendarDate['TYPE'] == \Service\CalendarDetail::WORK_TYPE) {
                if ($first) {
                    $first = false;
                } else {
                    return \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
                }
            }
        }
        throw new \Exception('date not found in CalendarDates');
    }

    /**
     * @param TaskCalendarInterface $task
     * @param $date
     * @param bool $first
     * @throws \Exception
     * @return \DateTime
     */
    public function getNextWorkingDayByTask($task, $date, $first = false)
    {
        $calendarDates = $this->getDatesCalendarNextMonth($task, $date);
        return $this->getNextWorkingDay($calendarDates, $first);
    }

    /**
     * @param TaskCalendarInterface $task
     * @param \DateTime $date
     * @param bool $first
     * @throws \Exception
     * @return \DateTime
     */
    public function getPreviousWorkingDayTask($task, $date, $first = false)
    {
        $calendarDates = $this->getDatesCalendarPreviousMonth($task, $date);
        return $this->getPreviousWorkingDay($calendarDates, $first);
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task | \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $task
     * @param $setAllocation
     * @return array
     */
    public function getDates($task, $setAllocation = true)
    {
        $dateRestriction = $this->getRestriction($task)->getDatesBeginAndEnd($setAllocation);
        return $dateRestriction;
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $taskPrecursor
     */
    public function setDatesTaskRelationship($taskPrecursor)
    {
        if ($taskPrecursor instanceof TaskCalendarInterface) {
            $taskRelationshipInverses = $taskPrecursor->getTaskRelationshipInverse();
            if (!empty($taskRelationshipInverses)) {
                /** @var Entities\TaskRelationship $taskRelationshipInverse */
                foreach ($taskRelationshipInverses as $taskRelationshipInverse) {
                    if ($taskRelationshipInverse->getType() == TaskPrecursor::PRECURSOR_TYPE) {
                        $task = $taskRelationshipInverse->getTask();
                        $task->setNeedScheduleCalcDates(true);
                        $this->setDates($task);
                    }
                }
            }
        }
    }

    /**
     * @param TaskCalendarInterface $task
     */
    public function setDate($task)
    {
        list($dateBegin, $dateEnd) = $this->getDatesBeginAndEnd($task);
        $task->setEstimatedBeginDate($dateBegin);
        $task->setEstimatedEndDate($dateEnd);
        $task->setInCalc('N');

    }

    /**
     * @param Entities\Task | Entities\Project $task
     * @param \DateTime $dateBeginMax
     * @param \DateTime $dateEndMin
     * @return array
     */
    public function setDateByChildren($task, \DateTime $dateBeginMax = null, \DateTime $dateEndMin = null)
    {
        if ($task->needScheduleCalcDates()) {
            $taskBegin = null;
            $taskEnd = null;

            list($dateBeginParent, $dateEndParent) = $this->getDatesBeginAndEnd($task);

            $task->setEstimatedBeginDate($dateBeginParent);
            $task->setEstimatedEndDate($dateEndParent);
            $hasChildren = $task->hasChildren();
            if ($hasChildren) {
                $children = $task->children();
                foreach ($children as $child) {
                    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $child */
                    if (!$child->isContainer() || $child->hasChildren()) {
                        list($dateBeginMax, $dateEndMin) = $this->getRestriction($child)->getDatesMinAndMax($dateBeginMax, $dateEndMin);
                        $child->setInCalc('N');
                    }
                }
                $taskBegin = $dateBeginMax;
                $taskEnd = $dateEndMin;

                foreach ($children as $child) {
                    if ($child->usesDataControl() && (!$child->isContainer() || $child->hasChildren())) {
                        if($child->hasChildren() &&
                            !in_array($child->getDateExceptionType(),RestrictionFactory::$ARRAY_RESTRICTION_PARENT)){

                            $child->setDateExceptionType(RestrictionFactory::AS_SOON_AS_POSSIBLE);
                            $this->entityManager->persist($child);
                        }
                        if($child->hasChildren()){
                            list($dateBegin, $dateEnd) = $this->setDateByChildren($child);
                        }else{
                            list($dateBegin, $dateEnd) = $this->getDatesBeginAndEnd($child);
                        }
                        if (($dateBegin < $taskBegin || is_null($taskBegin)) && !is_null($dateBegin)) {

                            $taskBegin = $dateBegin;
                        }
                        if (($dateEnd > $taskEnd || is_null($taskEnd)) && !is_null($dateEnd)) {
                            $taskEnd = $dateEnd;
                        }
                    }
                }

                if ($taskBegin) {
                    $task->setEstimatedBeginDate(clone $taskBegin);
                    $task->setEstimatedEndDate(clone $taskEnd);
                    $task->setDuration($this->setDuration($task));
                    $task->setNeedScheduleCalcDates(false);
                    $task->setInCalc('N');
                    $this->entityManager->persist($task);
                }
                if (!$task->getTreeParent()) {
                    $this->setChildren($task);
                }

                $task->setDateBeginMax($dateBeginMax);
                $task->setDateEndMin($dateEndMin);

                return array($taskBegin, $taskEnd, $dateBeginMax, $dateEndMin);

            }
        } else {
            $taskBegin = $task->getDateBeginMax();
            $taskEnd = $task->getDateEndMin();
        }
        return array($taskBegin, $taskEnd, $dateBeginMax, $dateEndMin);
    }

    /**
     * @param $calendar
     * @param $commitments
     * @return array
     */
    public function mergeCommitments($calendar, $commitments)
    {
//        $commitmentsNext = $calendar->getCommitments();
//        if (!empty($commitmentsNext)){
//            $commitments = array_merge($commitments, $commitmentsNext->getValues());
//        }
        return $commitments;
    }

    /**
     * @param $duration
     * @param  TaskCalendarInterface $task
     * @return int
     */
    public function getDurationInMilliseconds($duration, $task)
    {

        $duration = strtoupper($duration);
        $daySeparator = strpos($duration, 'D');
        if ($daySeparator) {
            $durationTotal = Utilities::timeToSeconds(substr($duration, strpos($duration, 'D') + 1));
            $durationTotal += $this->getDurationByNumberDaysAndDateBegin($task, clone $task->getEstimatedBeginDate(),
                intval(substr($duration, 0, strpos($duration, 'D'))));
        } else {
            $durationTotal = Utilities::timeToSeconds(substr($duration, strpos($duration, 'D')));
        }
        if (!$durationTotal) {
            $durationTotal = $this->getDurationByNumberDaysAndDateBegin($task, clone $task->getEstimatedBeginDate(),
                intval($duration));
        }
        return $durationTotal;
    }

    /**
     * @param  \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task |  \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $task
     * @return \DateTime
     */

    public function setDuration($task)
    {
        $begin = clone $task->getEstimatedBeginDate();
        $end = clone $task->getEstimatedEndDate();

        return $this->getDurationByDateBeginAndDateEnd($task, $begin, $end);
    }

    public function setProjectTop(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $project, EntityManager $entityManager)
    {
        $projectParent = $project->getParent();
        if ($projectParent) {
            $beginProject = $project->getEstimatedBeginDate();
            $endProject = $project->getEstimatedEndDate();
            $beginProjectParent = $projectParent->getEstimatedBeginDate();
            $endProjectParent = $projectParent->getEstimatedEndDate();
            $persist = false;
            if ($endProject > $endProjectParent) {
                $projectParent->setEstimatedEndDate($endProject);
                $persist = true;
            }
            if ($beginProject < $beginProjectParent) {
                $projectParent->setEstimatedBeginDate($beginProject);
                $persist = true;
            }
            if ($persist) {
                $projectParent->setDuration($this->setDuration($projectParent));
                $projectParent->setInCalc('N');
                $entityManager->persist($projectParent);
                $entityManager->flush();
            }
        }
    }

    /**
     * @param $task
     * @return \Service\Calendar\Restriction\Strategy\RestrictionStrategyAbstract
     */
    public function getRestriction(TaskCalendarInterface $task)
    {
        if ($task->getRestriction() === null) {
            $task->setRestriction(RestrictionFactory::factory($task, $this));
        }
        return $task->getRestriction();
    }

    /**
     * @param TaskCalendarInterface $task
     * @param $setAllocation
     * @return array
     */
    public function getDatesBeginAndEnd(TaskCalendarInterface $task, $setAllocation = true)
    {
        return $this->getDates($task, $setAllocation);
    }

    public function getDatesPrecursors(TaskCalendarInterface $task, $setAllocation = true)
    {
        $allDates = array();
        if ($task->hasTaskRelationship()) {
            $taskRelationships = $task->getTaskRelationship();
            /**
             * @var Task $taskRelationship
             */
            foreach ($taskRelationships as $taskRelationship) {
                if ($taskRelationship->getType() == TaskPrecursor::PRECURSOR_TYPE) {
                    $precursor = PrecursorFactory::factory($taskRelationship, $this);
                    array_push($allDates, $precursor->getDatesBeginAndEnd($setAllocation));
                }
            }
        }
        return $allDates;
    }

    /**
     * @param $task
     */
    public function setChildren(TaskCalendarInterface $task)
    {
        $directChildren = $task->directChildren();
        /**@var TaskCalendarInterface[] $directChildren */
        foreach ($directChildren as $child) {
            if ($child->usesDataControl() && !$child->isIssue()) {
                if($child->isContainer()) {
                    $this->setDateByChildren($child);
                }else{
                    $this->setDate($child);
                }
                $child->setNeedScheduleCalcDates(false);
                $child->setInCalc('N');
                $this->entityManager->persist($child);
                $this->setChildren($child);
            }
        }
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task
     */
    public function setPreviousTimeTask($task)
    {
        $previousTime = $this->getPreviousTime($task, $task->getDurationPrecursor());
        /** @var Entities\TaskRelationship[] $taskRelationshipInverses */
        $taskRelationshipInverses = InstanceProvider::getRepository()->getTaskRelationship()
            ->findBy(array('taskRelationship'=>$task, 'type'=>TaskPrecursor::PRECURSOR_TYPE,'typePrecursor'=> PrecursorFactory::END_TO_START));
        foreach ($taskRelationshipInverses as $taskRelationshipInverse) {
                $taskRelation = $taskRelationshipInverse->getTask();
                $taskRelation->setPreviousTime($previousTime);
                $this->setPreviousTimeTask($taskRelation);
        }

    }

    /**
     * @param TaskCalendarInterface $task
     * @param $previousTimePrecursor
     * @return int
     */
    public function getPreviousTime($task, $previousTimePrecursor)
    {
        $calendarDates = $this->getDatesCalendarNextMonth($task, $task->getEstimatedBeginDate());
        $duration = intval($previousTimePrecursor) + intval($task->getDuration());
        foreach ($calendarDates as $calendarDate) {
            $dayHours = intval($calendarDate['DAILY_HOURS']);
            if ($dayHours <= $duration) {
                $duration -= $dayHours;
            } else {
                return $duration;
            }
        }
        return null;
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task
     * @return int
     */
    private function getPreviousTimePrecursor($task)
    {
        $previousTime = 0;
        $tasksRelationship = $task->getTaskRelationship();
        if ($tasksRelationship) {
            foreach ($tasksRelationship as $taskRelationship) {
                if ($taskRelationship->getTypePrecursor() == PrecursorFactory::END_TO_START &&
                    $taskRelationship->getType() == TaskPrecursor::PRECURSOR_TYPE
                ) {
                    $taskPrecursor = $taskRelationship->getTaskRelationship();
                    if ($taskPrecursor->getPreviousTime() > $previousTime) {
                        $previousTime = $taskPrecursor->getPreviousTime();
                    }
                }
            }
        }
        return $previousTime;
    }

    public function getDatesCalendarNextMonth($task, \DateTime $begin)
    {
        $begin = clone $begin;
        $beginDate = $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $endDate = $begin->modify('+2 month')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        return $this->calendarService->findCalendarDatesForCalculate($task, $beginDate, $endDate);

    }

    public function getDatesCalendarPreviousMonth($task, \DateTime $begin)
    {
        $begin = clone $begin;
        $endDate = $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $beginDate = $begin->modify('-2 month')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        return $this->calendarService->findCalendarDatesForCalculate($task, $beginDate, $endDate);

    }

    /**
     * @param TaskCalendarInterface $task
     * @param \DateTime $begin
     * @param $first
     * @param $duration
     * @param bool $latency
     * @param bool $setAllocation
     * @return int
     */
    public function getEndDateByCalendar($task, \DateTime $begin, $first, $duration, $latency = false, $setAllocation = true)
    {
        $recursion = true;
        if ($duration) {
            $calendarDates = $this->getDatesCalendarNextMonth($task, $begin);
            foreach ($calendarDates as $calendarDate) {
                $dayHours = intval($calendarDate['DAILY_HOURS']);
                $begin = \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
                if (!$latency && $setAllocation && !$task->isContainer()) {
                    $daily = $duration > $dayHours ? $dayHours : $duration;
                    if ($first) {
                        $this->allocationService->setDailyAllocation($task, $begin, $daily - $task->getDurationPrecursor());
                        $first = false;
                    } else {
                        $this->allocationService->setDailyAllocation($task, $begin, $daily);
                    }
                }
                $duration -= $dayHours;
                if ($latency) {
                    if ($duration < 0) {
                        $recursion = false;
                        break;
                    }
                } else {
                    if ($duration <= 0) {
                        $recursion = false;
                        break;
                    }
                }
            }
            if ($recursion) {
                $begin = $this->getEndDateByCalendar($task, $begin, $first, $duration, $latency, $setAllocation);
            }
        }
        return $begin;


    }

    /**
     * @param TaskCalendarInterface $task
     * @param \DateTime $end
     * @param $first
     * @param $duration
     * @param bool $latency
     * @param bool $setAllocation
     * @return int
     */
    protected function getBeginDateByCalendar($task, \DateTime $end, $first, $duration, $latency = false, $setAllocation = true)
    {
        $recursion = true;
        $calendarDates = array_reverse($this->getDatesCalendarPreviousMonth($task, $end));
        foreach ($calendarDates as $calendarDate) {
            $dayHours = intval($calendarDate['DAILY_HOURS']);
            $end = \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
            if (!$latency && $setAllocation && !$task->isContainer()) {
                $daily = $duration > $dayHours ? $dayHours : $duration;
                if ($first) {
                    $this->allocationService->setDailyAllocation($task, $end, $daily - $task->getDurationPrecursor());
                    $first = false;
                } else {
                    $this->allocationService->setDailyAllocation($task, $end, $daily);
                }
            }
            $duration -= $dayHours;
            if ($latency) {
                if ($duration < 0) {
                    $recursion = false;
                    break;
                }
            } else {
                if ($duration <= 0) {
                    $recursion = false;
                    break;
                }
            }
        }
        if ($recursion) {
            $end = $this->getBeginDateByCalendar($task, $end, $first, $duration, $latency, $setAllocation);
        }
        return $end;

    }

    protected function getDurationByNumberDaysAndDateBegin($task, \DateTime $begin, $numberDays, $duration = 0)
    {
        $calendarDates = $this->getDatesCalendarNextMonth($task, $begin);
        foreach ($calendarDates as $calendarDate) {
            $dayHours = intval($calendarDate['DAILY_HOURS']);
            $begin = \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
            if ($calendarDate['TYPE'] == CalendarDetail::WORK_TYPE) {
                $duration += $dayHours;
                $numberDays--;
            }
            if ($numberDays <= 0) {
                return $duration;
            }
        }
        return $this->getDurationByNumberDaysAndDateBegin($task, $begin->modify('+1 day'), $numberDays, $duration);
    }

    protected function getDurationByDateBeginAndDateEnd(TaskCalendarInterface $task, \DateTime $begin, \DateTime $end)
    {
        $beginDate = $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $endDate = $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $calendarDates = $this->calendarService->findCalendarDatesForCalculate($task, $beginDate, $endDate);
        $duration = 0;
        $lastTime = $this->calendarService->getLastTaskChildren($task);
        $lastDateIndex = count($calendarDates)-1;
        foreach ($calendarDates as $index => $calendarDate) {
            if ($calendarDate['TYPE'] == CalendarDetail::WORK_TYPE) {
                $dayHours = intval($calendarDate['DAILY_HOURS']);
                if($lastTime && $lastDateIndex == $index){
                    $duration += $lastTime;
                }else {
                    $duration += $dayHours;
                }
            }

        }
        return $duration;
    }

    public function getDurationInDays($task, \DateTime $begin, \DateTime $end)
    {
        $beginDate = $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $endDate = $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $calendarDates = $this->calendarService->findCalendarDatesForCalculate($task, $beginDate, $endDate);
        $numberDays = 0;
        foreach ($calendarDates as $calendarDate) {
            if ($calendarDate['TYPE'] == CalendarDetail::WORK_TYPE) {
                $numberDays++;
            }
        }
        return $numberDays . 'D';
    }

    public function calculationBasedStart(TaskCalendarInterface $task, \DateTime $begin, $duration, TaskCalendarInterface $precursor = null,
                                          $precursorType = null, TaskCalendarInterface $parent = null, $exception, $exceptionDate,
                                          $latency = 0)
    {
        $task->setEstimatedBeginDate($begin);
        $duration = $this->getDurationInMilliseconds($duration, $task);
        $task->setDuration($duration);
        if ($latency) {
            $latency = $this->getDurationInMilliseconds($latency, $task);
        } else {
            $latency = 0;
        }
        $task->setLatency($latency);
        $previousTime = null;
        if ($precursor) {
            if ($precursorType == PrecursorFactory::END_TO_START && $exception == RestrictionFactory::AS_SOON_AS_POSSIBLE) {
                $previousTime = $this->getPreviousTime($precursor, $precursor->getDurationPrecursor());
            }
        }
        $task->setDurationPrecursor($previousTime);
        $begin = $this->getEndDateByCalendar($task, $begin, false, $task->getLatency() + $previousTime, true);
        $task->setEstimatedBeginDate($begin);
        list($begin, $end) = $this->getRestriction($task)->getDatesBeginAndEnd(false);
        return array($begin, $end, $exception, $exceptionDate);
    }


    public function calculationBasedEnd(TaskCalendarInterface $task, \DateTime $end, $duration, TaskCalendarInterface $precursor = null,
                                        $precursorType = null, TaskCalendarInterface $parent = null, $exception, $exceptionDate,
                                        $latency = 0)
    {
        $task->setEstimatedBeginDate($end);
        $duration = $this->getDurationInMilliseconds($duration, $task);
        $task->setDuration($duration);
        $previousTime = 0;
        if ($latency) {
            $latency = $this->getDurationInMilliseconds($latency, $task);
        } else {
            $latency = 0;
        }
        $task->setLatency($latency);
        if ($precursor) {
            if ($precursorType == PrecursorFactory::END_TO_START) {
                $previousTime = $this->getPreviousTime($precursor, $precursor->getDurationPrecursor());
            }
        }
        $task->setDurationPrecursor($previousTime);
        list($begin, $end) = $this->getRestriction($task)->getDatesBeginAndEnd(false);
        return array($begin, $end, $exception, $exceptionDate);
    }

    public function getEndDateByCalendarInFrontEnd($task, \DateTime $begin, $duration)
    {
        $calendarDates = $this->getDatesCalendarNextMonth($task, $begin);
        foreach ($calendarDates as $calendarDate) {
            $dayHours = intval($calendarDate['DAILY_HOURS']);
            $begin = \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
            $duration -= $dayHours;
            if ($duration <= 0) {
                return $begin;
            }
        }
        return $this->getEndDateByCalendarInFrontEnd($task, $begin, $duration);
    }

    public function getBeginDateByCalendarInFrontEnd($task, \DateTime $end, $duration)
    {
        $calendarDates = array_reverse($this->getDatesCalendarPreviousMonth($task, $end));
        foreach ($calendarDates as $calendarDate) {
            $dayHours = intval($calendarDate['DAILY_HOURS']);
            $end = \Zeedhi\Framework\ORM\DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $calendarDate['DATA'] . ' 00:00:00');
            $duration -= $dayHours;
            if ($duration <= 0) {
                return $end;
            }
        }
        return $this->getBeginDateByCalendarInFrontEnd($task, $end, $duration);
    }

    public function dateBeginEqualToUp($begin, TaskCalendarInterface $task, TaskCalendarInterface $parent = null,
                                       TaskCalendarInterface $precursor = null, $precursorType, $previousTime)
    {
        if ($precursor) {
            if ($precursorType == PrecursorFactory::END_TO_START) {
                if ($previousTime && $previousTime != 0) {
                    $date = $this->getNextWorkingDayByTask($precursor, $precursor->getEstimatedendDate());
                } else {
                    $date = $this->getNextWorkingDayByTask($precursor, $precursor->getEstimatedendDate(), true);
                }
                $return = Utilities::compareTwoDates($begin, $date);
            } else if ($precursorType == PrecursorFactory::START_TO_START) {
                $return = Utilities::compareTwoDates($begin, $precursor->getEstimatedBeginDate());
            } else {
                $return = true;
            }
        } else if ($parent) {
            $return = Utilities::compareTwoDates($begin, $parent->getEstimatedBeginDate());
        } else {
            $return = Utilities::compareTwoDates($begin, $task->getProject()->getEstimatedBeginDate());
        }
        return $return;
    }
}